-- itsymm: the no-nonsense, movable, scalable, square minimap.

-- default stuff
local def = { version = 40000.1, align = "TOPRIGHT", x = -25, y = -25, scale = 0.8, locked = true }
local logo, db = "|cffffffffitsy|cff44ccffmm|r", nil;

-- scaling variables
local increment, minimum, maximum = 0.01, 0.2, 2.0

-- initialize addon
local function FortyTwo(reset)
	if not ITSYDB or not ITSYDB.version or ITSYDB.version < 30300.3 or reset then
		ITSYDB = def; MinimapCluster:ClearAllPoints()
	end
	db = ITSYDB
	
	-- start-up positions, etc.
	Minimap:ClearAllPoints()
	Minimap:SetPoint(db.align, UIParent, db.align, db.x, db.y)
	Minimap:SetScale(db.scale)
	Minimap:SetMovable(true)
	-- pixel border
	Minimap.background = Minimap:CreateTexture(nil,"BACKGROUND")
	Minimap.background:SetTexture(0,0,0,1)
	Minimap.background:SetPoint("CENTER",Minimap,"CENTER")
	Minimap.background:SetWidth(Minimap:GetWidth()+2)
	Minimap.background:SetHeight(Minimap:GetHeight()+2)
end

local itsy = CreateFrame'frame'
itsy:RegisterEvent'ADDON_LOADED'
itsy:SetScript("OnEvent", function(self,_,addon)
	if addon == "ItsyMM" then
		if not ITSYDB then print(logo, "first run. Type /itsymm for info.") end
		self:SetScript("OnEvent", nil)
		FortyTwo()
	end
end)

function GetMinimapShape() return "SQUARE" end
Minimap:SetMaskTexture'Interface/Buttons/WHITE8X8'

-- itsy slash command
SLASH_ITSYMM1 = "/itsymm"
function SlashCmdList.ITSYMM(msg)
	if msg == "reset" then
		FortyTwo(true)
		print(logo, "settings reset.")
	elseif msg == "lock" then
		db.locked = not db.locked
		print(logo, db.locked and "locked." or "unlocked.")
	elseif msg:match'scale [%d%.]+' then
		db.scale = msg:match'scale ([%d%.]+)'
		Minimap:SetScale(db.scale)
	else
		print(format("%s: the no-nonsense, movable, scalable, square minimap.", logo))
		print(format("|cffcccccc %s lock|r - %s dragging of the minimap.", SLASH_ITSYMM1, db.locked and "|cff77ff77unlock|r" or "|cffff7777lock|r"))
		print(format("|cffcccccc %s scale %.2f |r- shift-mousewheel can also be used.", SLASH_ITSYMM1, db.scale))
		print(format("|cffcccccc %s reset|r - reset itsymm to its default state.", SLASH_ITSYMM1))
	end
end

-- hide crap
for _,f in pairs{
	GameTimeFrame,	-- this is the calendar/invite icon
	MiniMapBattlefieldBorder,		MiniMapLFGFrameBorder,
	MiniMapMailBorder,				MiniMapMeetingStoneBorder,
	MiniMapMeetingStoneFrame,		MiniMapTrackingBackground,
	MiniMapTrackingButtonBorder,	MiniMapVoiceChatFrame,
	MiniMapWorldMapButton,			MinimapBorder,
	MinimapBorderTop,				MinimapNorthTag,
	MinimapToggleButton,			MinimapZoneTextButton,
	MinimapZoomIn,					MinimapZoomOut,
} do f.Show = f.Hide f:Hide() end

-- enable mousewheel zoom/scale
Minimap:EnableMouseWheel(true)
Minimap:SetScript("OnMouseWheel", function(self, z)
	if not db.locked then
		local scale = self:GetScale() - increment * z
		if scale > minimum and scale < maximum then
			self:SetScale(scale)
			db.scale = scale
		end
	else
		_G["Minimap_Zoom"..(z > 0 and "In" or "Out")]()
	end
end)

-- making minimap movable
local function OnMouseUp(self)
	db.align,_,_,db.x,db.y = self:GetPoint()
	self:SetScript("OnMouseUp", Minimap_OnClick)
	self:StopMovingOrSizing()
end
Minimap:SetScript("OnMouseDown", function(self, button)
	if not db.locked then
		self:SetScript("OnMouseUp", OnMouseUp)
		self:StartMoving()
	end
end)

-- tracking button
MiniMapTracking:ClearAllPoints()
MiniMapTracking:SetPoint("TOPRIGHT",Minimap,"TOPRIGHT",-1,-10)
MiniMapTrackingButton:SetHighlightTexture''
MiniMapTrackingIcon:SetTexCoord(0.07,0.93,0.07,0.93)

-- lfg button
QueueStatusMinimapButton:ClearAllPoints()
QueueStatusMinimapButton:SetPoint("BOTTOMLEFT",Minimap,"BOTTOMLEFT",0,0)
QueueStatusMinimapButton:SetHighlightTexture''

-- battlefield button
--MiniMapBattlefieldFrame:ClearAllPoints()
--MiniMapBattlefieldFrame:SetPoint("BOTTOMLEFT",Minimap,"BOTTOMLEFT",0,0)

-- you have mail button
MiniMapMailFrame:ClearAllPoints()
MiniMapMailFrame:SetPoint("BOTTOMRIGHT",Minimap,"BOTTOMRIGHT",0,0)
MiniMapMailIcon:SetTexture'Interface/Minimap/Tracking/Mailbox'
MiniMapMailFrame:SetScale(1.1)

-- move raid difficulty thingy
MiniMapInstanceDifficulty:ClearAllPoints()
MiniMapInstanceDifficulty:SetParent(Minimap)
MiniMapInstanceDifficulty:SetPoint("CENTER",Minimap,"TOPRIGHT",0,0)

-- move guild raid difficulty thingy
GuildInstanceDifficulty:ClearAllPoints()
GuildInstanceDifficulty:SetParent(Minimap)
GuildInstanceDifficulty:SetPoint("CENTER",Minimap,"TOPRIGHT",0,0)
