--CREDITS: Xrystal for nUI support code, seebs for line spacing code
--------------------------------------------------------------------

local skin = "Interface\\AddOns\\rActionButtonStyler\\media\\gloss.tga"		--the normal button skin texture
local hover = "Interface\\AddOns\\rActionButtonStyler\\media\\hover.tga"	--the texture when hovering your mouse over the button
local pushed = "Interface\\AddOns\\rActionButtonStyler\\media\\pushed.tga"	--the texture when pushing the button

local WFContainer = CreateFrame("Frame", "WFContainer", UIParent)
local db, SetSize, MoveWatchFrame, Appearance, FontChanges, line	--, WFWW_BF, itembtn
local nextline = 1
local nextbutton = 1
local LSM = LibStub("LibSharedMedia-3.0")
local widgetLists = AceGUIWidgetLSMlists
local GameTooltip = GameTooltip
local WatchFrameCollapseExpandButton = WatchFrameCollapseExpandButton
local positions = {"TOP", "TOPRIGHT", "TOPLEFT",
					"BOTTOM", "BOTTOMRIGHT", "BOTTOMLEFT",
					"RIGHT", "LEFT", "CENTER"}
local fontFlags = {"None", "Outline", "Monochrome Outline"}	--, "Monochrome"}
local wideFrame = GetCVar("watchFrameWidth")
local _,class = UnitClass("player")
local classColor


local defaults = {
	bg = "Solid",
	border = "Blizzard Tooltip",
	borderWidth = 16,
	inset = 5,
	anchorPoint = "TOPRIGHT",
	xOffset = "-15",
	yOffset = "-250",
	height = 500,
	bgColor = {r = .1, g = .1, b = .1, a = .6},
	borderColor = {r = 1, g = 1, b = 1},
	borderAlpha = .8,
	classBorder = false,
	font = "Friz Quadrata TT",
	fontSize = 12,
	fontFlag = "None",
	questSpacing = 10,
	lineSpacing = 4,
	disableShadow = false,
	scale = 1,
	hideFrame = false,
	instance = false,
	keybind = "",
}

local options = {
	name = "Who Framed Watcher Wabbit?",
	type = "group",
	args = {
		spacer = {
			name = " ",
			type = "description",
			width = "full",
			order = 0,
		},
		title1 = {
			name = "Size and Position",
			type = "header",
			order = .5,
		},
		anchor = {
			name = "Screen Anchor",
			desc = "What side of the screen you would like the WatchFrame located at. The tracker will also collapse in this direction.",
			type = "select",
			values = positions,
			get = function()
						for k, v in pairs(positions) do
							if db.anchorPoint == v then
								return k
							end
						end
					end,
			set = function(_, anchor)
						db.anchorPoint = positions[anchor]
						db.xOffset = "0"
						db.yOffset = "0"
						MoveWatchFrame()
					end,
			order = 1,
		},
		xOffset = {
			name = "X Offset",
			desc = "Offsets from the watch frame's anchor left/right",
			type = "input",
			get = function() return db.xOffset end,
			set = function(_,xOffset)
						if xOffset == nil or xOffset == "" then
							xOffset = "0"
						end
						db.xOffset = xOffset
						MoveWatchFrame()
					end,
			order = 2,
		},
		yOffset = {
			name = "Y Offset",
			desc = "Offsets from the watch frame's anchor up/down",
			type = "input",
			get = function() return db.yOffset end,
			set = function(_,yOffset)
						if yOffset == nil or yOffset == "" then
							yOffset = "0"
						end
						db.yOffset = yOffset
						MoveWatchFrame()
					end,
			order = 3,
		},
		scale = {
			name = "WatchFrame scale",
			desc = "The scale of the entire tracker frame.",
			type = "range",
			min = .5,
			max = 2,
			step = .05,
			get = function() return db.scale end,
			set = function(_, scale) 
						db.scale = scale
						SetSize()
					end,
			order = 4,
		},
		height = {
			name = "WatchFrame height",
			desc = "The maximum height of the tracker frame. A larger height can display more quests.",
			type = "range",
			min = 200,
			max = 1000,
			step = 5,
			get = function() return db.height end,
			set = function(_,height)
						db.height = height
						SetSize()
					end,
			order = 5,
		},
		spacer1 = {
			name = " ",
			type = "description",
			width = "full",
			order = 5.4,
		},
		title2 = {
			name = "Font Options",
			type = "header",
			order = 5.5,
		},
		font = {
			name = "Font",
			desc = "Select the font for the watch frame.",
			type = "select",
			dialogControl = "LSM30_Font",
			values = widgetLists.font,
			get = function()
					return db.font
				end,
			set = function(self, font)
					db.font = font
					nextline = 1
					WatchFrame_Update()
				end,
			order = 6,
		},
		fontSize = {
			name = "Text Size",
			desc = "Change the font size of the watch frame.",
			type = "range",
			min = 6,
			max = 24,
			step = 1,
			get = function() return db.fontSize end,
			set = function(_,size) 
						db.fontSize = size
						nextline = 1
						WatchFrame_Update()
					end,
			order = 7,
		},
		fontFlag = {
			name = "Font Flag",
			desc = "Set how to alter the watch frame's font.",
			type = "select",
			values = fontFlags,
			get = function()
						for k, v in pairs(fontFlags) do
							if db.fontFlag == v then
								return k
							end
						end
					end,
			set = function(_,key)
						db.fontFlag = fontFlags[key]
						nextline = 1	--start over from the beginning
						WatchFrame_Update()
					end,
			order = 8,
		},
		disableShadow = {
			name = "Disable Font Shadow",
			desc = "If checked, the shadow effect on each line will be hidden.",
			type = "toggle",
			get = function() return db.disableShadow end,
			set = function()
						db.disableShadow = not db.disableShadow
						nextline = 1
						FontChanges()
					end,
			order = 8.5,
		},
		questSpacing = {
			name = "Quest Spacing",
			desc = "Change the spacing between quests.",
			type = "range",
			min = 0,
			max = 12,
			step = 1,
			get = function() return db.questSpacing end,
			set = function(_,size)
						db.questSpacing = size
						WATCHFRAME_QUEST_OFFSET = size
						WatchFrame_Update()
					end,
			order = 8.51,
		},
		lineSpacing = {
			name = "Line Spacing",
			desc = "Change the spacing between lines.",
			type = "range",
			min = 0,
			max = 10,
			step = 1,
			get = function() return db.lineSpacing end,
			set = function(_,size)
						db.lineSpacing = size
						WATCHFRAME_LINEHEIGHT = db.fontSize + size
						WatchFrame_Update()
					end,
			order = 8.52,
		},
		spacer2 = {
			name = " ",
			type = "description",
			width = "full",
			order = 8.6,
		},
		title3 = {
			name = "Texture Options",
			type = "header",
			order = 8.7,
		},
		bg = {
			name = "Background",
			desc = "Texture to use as the tracker's background.",
			type = "select",
			dialogControl = "LSM30_Background",
			values = widgetLists.background,
			get = function()
					return db.bg
				end,
			set = function(self, bg)
					db.bg = bg
					Appearance()
				end,
			order = 9,
		},
		bgColor = {
			name = "Background Color/Alpha",
			desc = "The background color and transparency.\n\nIf you wish to use a custom background texture and do not want it colored, set this to white.",
			type = "color",
			hasAlpha = true,
			width = "double",
			get = function() return db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a end,
			set = function(_,r,g,b,a)
					db.bgColor.r = r
					db.bgColor.g = g
					db.bgColor.b = b
					db.bgColor.a = a
					Appearance()
				end,
			order = 10,
		},
		border = {
			name = "Border",
			desc = "Texture for the watch frame's border.",
			type = "select",
			dialogControl = "LSM30_Border",
			values = widgetLists.border,
			get = function()
					return db.border
				end,
			set = function(self, border)
					db.border = border
					Appearance()
				end,
			order = 11,
		},
		borderColor = {
			name = "Border Color",
			desc = "Color of the watch frame's border.",
			type = "color",
			disabled = function() return db.classBorder end,
			get = function() return db.borderColor.r, db.borderColor.g, db.borderColor.b end,
			set = function(_,r,g,b)
						db.borderColor.r = r
						db.borderColor.g = g
						db.borderColor.b = b
						Appearance()
					end,
			order = 12,
		},
		classBorder = {
			name = "Class Colored Border",
			desc = "Color the watch frame's border by your class.",
			type = "toggle",
			get = function() return db.classBorder end,
			set = function()
						db.classBorder = not db.classBorder
						Appearance()
					end,
			order = 12.5,
		},
		borderAlpha = {
			name = "Border Alpha",
			desc = "Transparency of the border.",
			type = "range",
			min = 0,
			max = 1,
			step = .05,
			get = function() return db.borderAlpha end,
			set = function(_, borderAlpha) 
						db.borderAlpha = borderAlpha
						Appearance()
					end,
			order = 13,
		},
		borderWidth = {
			name = "Border Width",
			desc = "Width of the border.",
			type = "range",
			min = 1,
			max = 24,
			step = .5,
			get = function() return db.borderWidth end,
			set = function(_, borderWidth) 
						db.borderWidth = borderWidth
						Appearance()
					end,
			order = 13.5,
		},
		inset = {
			name = "Inset",
			desc = "The inset of the background from the border texture.",
			type = "range",
			min = 0,
			max = 10,
			step = .5,
			get = function() return db.inset end,
			set = function(_, inset) 
						db.inset = inset
						Appearance()
					end,
			order = 14,
		},
		spacer3 = {
			name = " ",
			type = "description",
			width = "full",
			order = 14.4,
		},
		title4 = {
			name = "Miscellaneous",
			type = "header",
			order = 14.5,
		},
		keybind = {
			name = "Keybinding",
			desc = "Key to minimize/maximize the tracker.",
			type = "keybinding",
			get = function() return db.keybind end,
			set = function(_,key)
						SetOverrideBinding(WFContainer, false, db.keybind, nil)	--clear old binding
						if key ~= "" then	--set new
							SetOverrideBinding(WFContainer, false, key, "CLICK WatchFrameCollapseExpandButton:LeftButton")
						end
						db.keybind = key
					end,
			order = 15,
		},
		hideFrame = {
			name = "Hide frame when empty",
			desc = "Hide the watch frame when no quests or achievements are being tracked.",
			type = "toggle",
			get = function() return db.hideFrame end,
			set = function()
						db.hideFrame = not db.hideFrame
						if db.hideFrame and GetNumTrackedAchievements() == 0 and GetNumQuestWatches() == 0 then
							WFContainer:SetAlpha(0)
							WatchFrameCollapseExpandButton:EnableMouse(false)
						else
							WFContainer:SetAlpha(1)
							WatchFrameCollapseExpandButton:EnableMouse(true)
						end
					end,
			order = 15.5,
		},
		instance = {
			name = "Collapse in instance",
			desc = "Automatically collapses the watch frame when entering an instance.",
			type = "toggle",
			get = function() return db.instance end,
			set = function()
						db.instance = not db.instance
						if db.instance then
							WFContainer:RegisterEvent("PLAYER_ENTERING_WORLD")
						else
							WFContainer:UnregisterEvent("PLAYER_ENTERING_WORLD")
						end
					end,
			order = 15.7,
		},
		spacer4 = {
			name = " ",
			type = "description",
			width = "full",
			order = 16,
		},
		divider = {
			name = "Profile Management",
			type = "header",
			order = 17,
		},
		charSpec = {
			name = "Character specific settings",
			desc = "Have this character use their own profile.  If checked, any changes made will not affect other characters.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
			type = "toggle",
			width = "full",
			confirm = true,
			get = function() return WFWWPCDB.charSpec end,
			set = function()
						WFWWPCDB.charSpec = not WFWWPCDB.charSpec
						ReloadUI()
					end,
			order = 18,
		},
		copyProfile = {
			name = "Copy from Default",
			desc = "Copy all settings from the default, global profile to this character's profile.  This will not effect other characters' specific profiles.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
			type = "execute",
			confirm = true,
			disabled = function() return not WFWWPCDB.charSpec end,
			func = function()
						WFWWPCDB = WFWWDB
						WFWWPCDB.charSpec = true
						ReloadUI()
					end,
			order = 19,
		},
		resetProfile = {
			name = "Profile Reset",
			desc = "Reset this profile back to the out-of-the-box settings.  If you reset the character specific profile, the global profile will be untouched, and vice versa.  This will not effect other character specific profiles.\n\n|c00E30016WARNING:|r Your UI will be reloaded in the process!",
			type = "execute",
			confirm = true,
			func = function()
						if WFWWPCDB.charSpec then
							WFWWPCDB = {charSpec = true}
						else
							WFWWDB = {}
						end
						ReloadUI()
					end,
			order = 20,
		},
	},
}

function MoveWatchFrame()
	WFContainer:ClearAllPoints()
	WFContainer:SetPoint(db.anchorPoint, UIParent, db.anchorPoint, db.xOffset, db.yOffset)
end

function SetSize()
	WFContainer:SetHeight(db.height)
	if wideFrame == "1" then
		WFContainer:SetWidth(350)
	else
		WFContainer:SetWidth(250)
	end
	WFContainer:SetScale(db.scale)
end

function Appearance()
	WFContainer:SetBackdrop({
			bgFile = LSM:Fetch("background", db.bg),
			edgeFile = LSM:Fetch("border", db.border),
			edgeSize = db.borderWidth,
			insets = {left=db.inset,right=db.inset,top=db.inset,bottom=db.inset}
		})
	WFContainer:SetBackdropColor(db.bgColor.r,db.bgColor.g,db.bgColor.b,db.bgColor.a)
	if db.classBorder then
		WFContainer:SetBackdropBorderColor(classColor.r,classColor.g,classColor.b,db.borderAlpha)
	else
		WFContainer:SetBackdropBorderColor(db.borderColor.r,db.borderColor.g,db.borderColor.b,db.borderAlpha)
	end
end

function FontChanges()
	local font = LSM:Fetch("font", db.font)
	WatchFrameTitle:SetFont(font, db.fontSize, db.fontFlag)
	for i = nextline, 50 do
		line = _G["WatchFrameLine"..i]
		if line then
			line.text:SetFont(font, db.fontSize, db.fontFlag)
			line.dash:SetFont(font, db.fontSize, db.fontFlag)
			if db.disableShadow then
				line.text:SetShadowColor(0,0,0,0)
				line.dash:SetShadowColor(0,0,0,0)
			else
				line.text:SetShadowColor(0,0,0,1)
				line.dash:SetShadowColor(0,0,0,1)
			end
		else
			nextline = i	--so we only have to change new lines during the hook
			break
		end
	end
end

local function SkinButtons()
	if IsAddOnLoaded("rActionButtonStyler") then
		local colorR,colorG,colorB = ActionButton1NormalTexture:GetVertexColor()
		for i = nextbutton, 15 do
			itembtn = _G["WatchFrameItem"..i]
			if itembtn then
				itembtn:SetHighlightTexture(hover)
				itembtn:SetPushedTexture(pushed)
				border = _G[itembtn:GetName().."NormalTexture"]
				border:SetAllPoints()
				border:SetTexture(skin)
				border:SetVertexColor(colorR, colorG, colorB)
			else
				nextbutton = i
				break
			end
		end
	end
end

local function SetUpContainer()
	--keep nUI from hogging the WatchFrame - thanks Xrystal!
	if IsAddOnLoaded("nUI") then
		nUI_ResetWatchFrame = function() end
		nUI_Movers:lockFrame(WatchFrame, false, nil)
		nUI_Options.movedWatchFrame = true
	end
	
	MoveWatchFrame()
	SetSize()
	Appearance()
	SkinButtons()
	if db.keybind ~= "" then
		SetOverrideBinding(WFContainer, false, db.keybind, "CLICK WatchFrameCollapseExpandButton:LeftButton")
	end	
	WATCHFRAME_QUEST_OFFSET = db.questSpacing
	WATCHFRAME_LINEHEIGHT = db.fontSize + db.lineSpacing
	
	hooksecurefunc("WatchFrame_Update", function()
		FontChanges()
		SkinButtons()

		if db.hideFrame then
			if GetNumTrackedAchievements() == 0 and GetNumQuestWatches() == 0 then
				WFContainer:SetAlpha(0)
				WatchFrameCollapseExpandButton:EnableMouse(false)
			else
				WFContainer:SetAlpha(1)
				WatchFrameCollapseExpandButton:EnableMouse(true)
			end
		end
	end)
	
	WatchFrame_Update()
	
	hooksecurefunc("WatchFrameLinkButtonTemplate_Highlight", function(self, enter)
		if enter then
			local id = self.index
			local watchType = self.type
			if watchType == "QUEST" then
				GameTooltip:SetOwner(self)
				GameTooltip:SetHyperlink(GetQuestLink(GetQuestIndexForWatch(id)))
				GameTooltip:Show()
			elseif watchType == "ACHIEVEMENT" then
				GameTooltip:SetOwner(self)
				GameTooltip:SetHyperlink(GetAchievementLink(id))
				GameTooltip:Show()			
			end
		else
			GameTooltip:Hide()
		end
	end)
	
	WatchFrame:SetParent(WFContainer)
	WatchFrame:SetClampedToScreen(false)
	WatchFrame:ClearAllPoints()
	WatchFrame.ClearAllPoints = function() end
	WatchFrame:SetPoint("TOPLEFT", 32,-2.5)
	WatchFrame:SetPoint("BOTTOMRIGHT", 4,0)
	WatchFrame.SetPoint = function() end

	WatchFrameTitle:SetParent(WFContainer)
	WatchFrameCollapseExpandButton:SetParent(WFContainer)
	WatchFrameCollapseExpandButton:Raise()	--let us click it even if header tex is too big
	WatchFrameTitle:Show()
	WatchFrameCollapseExpandButton:Show()
	WatchFrameTitle.Hide = function() end
	WatchFrameCollapseExpandButton.Hide = function() end
	WatchFrameCollapseExpandButton.Disable = function() end
	
	if WFWWPCDB.collapsed then	--remember if watchframe was collapsed on last logout
		WatchFrame_CollapseExpandButton_OnClick()	--default function called by OnClick
		WFContainer:SetHeight(30)
		WFContainer:SetWidth(150)
	end
	
	WatchFrameCollapseExpandButton:RegisterForClicks("AnyDown")
	WatchFrameCollapseExpandButton:SetScript("OnClick", function(self,btn)	--redo this to allow for modified clicks
		if IsShiftKeyDown() then
			ToggleAchievementFrame()
		elseif IsAltKeyDown() then
			InterfaceOptionsFrame_OpenToCategory("Who Framed Watcher Wabbit?")
		elseif btn == "RightButton" then
			ToggleFrame(QuestLogFrame)
		else
			WatchFrame_CollapseExpandButton_OnClick()	--default function called by OnClick
			if WatchFrame.userCollapsed then	--my original hook
				WFContainer:SetHeight(30)
				WFContainer:SetWidth(150)
				WFWWPCDB.collapsed = true
			else
				WFContainer:SetHeight(db.height)
				if wideFrame == "1" then
					WFContainer:SetWidth(350)
				else
					WFContainer:SetWidth(250)
				end
				WFWWPCDB.collapsed = false
			end
		end
	end)
	WatchFrameCollapseExpandButton:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_TOP")
			GameTooltip:AddLine("Who Framed Watcher Wabbit?",1,1,1)
			if WatchFrame.collapsed then
				GameTooltip:AddLine("Left-click to expand the objectives tracker.")
			else
				GameTooltip:AddLine("Left-click to minimize the objectives tracker.")
			end
			GameTooltip:AddLine("Right-click to toggle the quest log.")
			GameTooltip:AddLine("Shift-click to toggle the achievements window.")
			GameTooltip:AddLine("Alt-click to open the Who Framed Watcher Wabbit? options.")
			GameTooltip:Show()
		end)
	WatchFrameCollapseExpandButton:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

local function SetUpDB()
	if WFWWPCDB.charSpec then
		--set defaults if new charSpec DB
		for k,v in pairs(defaults) do
			if type(WFWWPCDB[k]) == "nil" then
				WFWWPCDB[k] = v
			end
		end
		db = WFWWPCDB
	else
		db = WFWWDB
	end
end

local function Initialize()
	WFWWDB = WFWWDB or {}
	WFWWPCDB = WFWWPCDB or {}
		if WFWWPCDB.charSpec == nil then
			WFWWPCDB.charSpec = false
		end
		if WFWWPCDB.collapsed == nil then
			WFWWPCDB.collapsed = false
		end
	for k,v in pairs(defaults) do
	    if type(WFWWDB[k]) == "nil" then
	        WFWWDB[k] = v
	    end
	end
	SetUpDB()
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Who Framed Watcher Wabbit?", options)
	WFContainer.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Who Framed Watcher Wabbit?", "Who Framed Watcher Wabbit?")
	SlashCmdList["WFWW"] = function() InterfaceOptionsFrame_OpenToCategory("Who Framed Watcher Wabbit?") end
	SLASH_WFWW1 = "/wfww"
	
	classColor = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[class] or RAID_CLASS_COLORS[class]
	if CUSTOM_CLASS_COLORS then
		CUSTOM_CLASS_COLORS:RegisterCallback(function()
				if db.classBorder then
					Appearance()
				end
			end)
	end
	
	if not db.instance then
		WFContainer:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
	WFContainer:RegisterEvent("QUEST_WATCH_UPDATE")
	WFContainer:RegisterEvent("QUEST_AUTOCOMPLETE")
	WFContainer:RegisterEvent("CVAR_UPDATE")
	WFContainer:SetScript("OnEvent", function(_,event,...)
			if event == "PLAYER_ENTERING_WORLD" then
				if IsInInstance() and not WFWWPCDB.collapsed then
				--if you go in the instance and the WF isn't collapsed...
				WatchFrameCollapseExpandButton:Click()	--collapse
				end
			elseif event == "CVAR_UPDATE" then
				local cvar,value = ...
				if cvar == "WATCH_FRAME_WIDTH_TEXT" then
					if not WatchFrame.userCollapsed then
						if value == "1" then
							WFContainer:SetWidth(350)
						else
							WFContainer:SetWidth(250)
						end
					end
					wideFrame = value
				end
			else
				local index = ...
				if event == "QUEST_AUTOCOMPLETE" or index == 0 then
					if WFWWPCDB.collapsed then
						WFContainer:SetHeight(db.height)
						if wideFrame == "1" then
							WFContainer:SetWidth(350)
						else
							WFContainer:SetWidth(250)
						end
						WFWWPCDB.collapsed = false
					end
				end
			end
		end)
end

WFContainer:RegisterEvent("PLAYER_ENTERING_WORLD")
local f = CreateFrame("Frame")	--need to create a slight delay to get rid of errors and glitches
f:Hide()
f.elapsed = 0
f:SetScript("OnUpdate", function(self, elapsed)
		f.elapsed = f.elapsed + elapsed
		if f.elapsed > .5 then
			SetUpContainer()
			f:Hide()
		end
	end)
WFContainer:SetScript("OnEvent", function()
		Initialize()
		f:Show()
	end)